notmember(_,[]).
notmember(X,[X|_]) :- fail.
notmember(X,[Y|L]) :- X #\= Y, notmember(X,L).

compose(N) :- 1 #< X, X #< N, 1 #< Y, Y #< N, N #= X * Y.

premier(N) :- compose(N), !, fail.
premier(_). 

pair(X) :- X #= 2 * _.
pair(X) :- X #= 2 * _ + 1, fail.

impair(X) :- X #= 2 * _ + 1.
impair(X) :- X #= 2 * _, fail.

nombre_pair([]) :- fail.
nombre_pair([X]) :- pair(X), !.
nombre_pair([_|L]) :- nombre_pair(L).

nombre_impair([]) :- fail.
nombre_impair([X]) :- impair(X), !.
nombre_impair([_|L]) :- nombre_impair(L).

somme_chiffres([],0).
somme_chiffres([X|L],N) :- somme_chiffres(L,NN), N #= X + NN.

produit_chiffres([],1).
produit_chiffres([X|L],N) :- produit_chiffres(L,NN), N #= X * NN.

tous_impairs([]).
tous_impairs([X|_]) :- pair(X), !, fail.
tous_impairs([_|L]) :- tous_impairs(L).

tous_inferieurs([]).
tous_inferieurs([_]).
tous_inferieurs([X,Y|L]) :- X #=< Y, tous_inferieurs([Y|L]).

pastous_inferieurs([]) :- fail.
pastous_inferieurs([_]) :- fail.
pastous_inferieurs([X,Y|_]) :- X #> Y, !.
pastous_inferieurs([_,Y|L]) :- pastous_inferieurs([Y|L]).

pasall_different([]) :- fail.
pasall_different([_]) :- fail.
pasall_different([X|L]) :- member(X,L), !.
pasall_different([_|L]) :- pasall_different(L).

tous_moins_que_4([]).
tous_moins_que_4([X|_]) :- X #> 4, !, fail.
tous_moins_que_4([_|L]) :- tous_moins_que_4(L).

pas_tous_moins_que_4([]) :- fail.
pas_tous_moins_que_4([X|_]) :- X #> 4, !.
pas_tous_moins_que_4([_|L]) :- pas_tous_moins_que_4(L).

pas_difference_1([]).
pas_difference_1([_]).
pas_difference_1([0|L]) :- notmember(1,L), pas_difference_1(L).
pas_difference_1([X|L]) :- X1 #= X-1, X2 #= X+1, notmember(X1,L), notmember(X2,L), pas_difference_1(L).

difference_1([]) :- fail.
difference_1([_]) :- fail.
difference_1([X|L]) :- X1 #= X-1, member(X1,L), !.
difference_1([X|L]) :- X2 #= X+1, member(X2,L), !.
difference_1([_|L]) :- difference_1(L).

retire(_,[],_) :- fail.
retire(X,[X|L],L) :- !.
retire(X,[Y|L],[Y|LL]) :- retire(X,L,LL).

existe_somme([],_) :- fail.
existe_somme([X|_],Liste2) :- retire(X,Liste2,Liste), member(Y,Liste), retire(Y,Liste,LL), member(Z,LL), X #= Y + Z.
existe_somme([_|L],Liste) :- existe_somme(L,Liste).

pas_somme(_,[]).
pas_somme(_,[_]).
pas_somme(X,[Y,Z]) :- X #\= Y + Z.
pas_somme(X,[Y,Z,T|L]) :- XY #= X - Y, notmember(XY,[Z,T|L]), pas_somme(X,[Z,T|L]).
pas_somme(X,[Y,Z,T|L]) :- X #< Y, pas_somme(X,[Z,T|L]).

existe_pas_somme([],_).
existe_pas_somme([X|L],Liste) :- retire(X,Liste,Liste2), pas_somme(X,Liste2), existe_pas_somme(L,Liste).

assertion(0,Liste) :- somme_chiffres(Liste,N), fd_prime(N).
assertion(1,Liste) :- produit_chiffres(Liste,N), impair(N).
assertion(2,Liste) :- tous_inferieurs(Liste).
assertion(3,Liste) :- fd_all_different(Liste).
assertion(4,Liste) :- tous_moins_que_4(Liste).
assertion(5,Liste) :- length(Liste,N), N #=< 6.
assertion(6,Liste) :- produit_chiffres(Liste,N), N #= 6 * _ + R, 0 #< R, R #< 6.
assertion(7,Liste) :- nombre_pair(Liste).
assertion(8,Liste) :- pas_difference_1(Liste).
assertion(9,Liste) :- existe_somme(Liste,Liste).

negassertion(0,Liste) :- somme_chiffres(Liste,N), fd_not_prime(N).
negassertion(1,Liste) :- produit_chiffres(Liste,N), pair(N).
negassertion(2,Liste) :- pastous_inferieurs(Liste).
negassertion(3,Liste) :- pasall_different(Liste).
negassertion(4,Liste) :- pas_tous_moins_que_4(Liste).
negassertion(5,Liste) :- length(Liste,N), N #> 6.
negassertion(6,Liste) :- produit_chiffres(Liste,N), N #= 6 * _.
negassertion(7,Liste) :- nombre_impair(Liste).
negassertion(8,Liste) :- difference_1(Liste).
negassertion(9,Liste) :- existe_pas_somme(Liste,Liste).

domaine([]).
domaine([X|L]) :- 0 #=< X, X #=< 9, domaine(L).

assertion_base([]) :- !, fail.
assertion_base([X|_]) :- X #> 0.

contraintes(_,[]).
contraintes(Liste,[N|L]) :- assertion(N,Liste), member(N,Liste), contraintes(Liste,L).
contraintes(Liste,[N|L]) :- negassertion(N,Liste), notmember(N,Liste), contraintes(Liste,L).

resout_taille(L,N) :- length(L,N), domaine(L), assertion_base(L), contraintes(L,[8,7,6,5,4,3,2,1,0,9]), fd_labeling(L).
resout_taille(L,N) :- nl, NN is N+1, resout_taille(L,NN).

resout(L) :- resout_taille(L,1).
